#!/usr/bin/env python
# -*- coding: utf-8 -*-
from builtins import str
import os
import re
import subprocess

import boto3
from botocore.exceptions import ClientError

from cloud_resources.error_handler import S3OperationError, InvalidInputError


def create_partition(partition_name):
    """
    Creates a partition with given name on the storage service: AWS s3.

    Parameters
    ----------
    partition_name: String
        Name of the partition (bucket in this case) to be created
    """
    if partition_name == '' or not partition_name:
        raise S3OperationError(
            "S3: No bucket name specified while creating bucket")
    try:
        client = boto3.client('s3')
        client.create_bucket(Bucket=partition_name)
    except ClientError as e:
        raise S3OperationError(str(e))


def write_object(full_file_path, obj_data, partition_name):
    """
    Writes a string object to the the given location on S3.

    Parameters
    ----------
    full_file_path: String
        Complete s3 path
    obj_data: String
        Object to be written on s3
    partition_name: String
        Name of bucket in which the object will be written
    """
    try:
        client = boto3.client('s3')
        client.put_object(Bucket=partition_name,
                          Body=obj_data, Key=full_file_path)
    except ClientError as e:
        raise S3OperationError("S3: Cannot write object to s3: " + str(e))


def get_object(location_key, partition_name):
    """
    Returns an object from a given partition for a given key (path).

    Parameters
    ----------
    location_key: String
        Full path to the partition (s3 bucket)

    Returns
    ------
    String: Fetched object
    """
    try:
        client = boto3.client('s3')
        response = client.get_object(Bucket=partition_name, Key=location_key)
    except ClientError as e:
        raise S3OperationError("Not able to get the files from S3: " + str(e))

    return response['Body'].read()


def get_directory_objects(partition_name, directory):
    """
    Given a directory name, returns the objects in a given partition.

    In AWS S3 infrastructure, partition will refer to a key path and directory
    will refer to a bucket.

    Parameters
    ----------
    partition_name: String
        Path of the partition
    directory_name: String
        Name of the directory (Key)

    Returns
    -------
    List
        List of objects with relative path from the partition name
    """
    if not directory.endswith('/'):
        directory += '/'
    object_keys = []
    try:
        s3 = boto3.resource('s3')
        bucket = s3.Bucket(partition_name)
        for s3_object in bucket.objects.filter(Prefix=directory):
            object_keys.append(re.sub(directory, '', s3_object.key))
    except ClientError as e:
        raise S3OperationError(str(e))
    return object_keys


def get_directory_objects_raw(partition_name, directory):
    """
    Returns all objects in a directory for a given partition name.

    Parameters
    ----------
    partition_name: String
        Path of the partition
    directory_name: String
        Name of the directory (Key)

    Returns
    -------
    Object
        Objects in the directory
    """
    if not directory.endswith('/'):
        directory += '/'

    s3 = boto3.resource('s3')
    bucket = s3.Bucket(partition_name)
    return bucket.objects.filter(Prefix=directory)

def copy_file_object(old_bucket_name, old_prefix, new_bucket_name, new_prefix):
    """
    Copy file(s) in both intra-bucket and inter-bucket 

    Parameters
    ----------
    old_bucket_name: string
        name of the bucket from which we have to copy files
    old_prefix: string
        In case of folder, it will be path of the folder from which we have
        to copy files, otherwise it will be actual path of the file
    new_bucket_name: string
        name of the bucket to which we have to copy files
    new_prefix: string
        path to which you want to copy files to
    """
    try:
        s3 = boto3.resource('s3')
        old_bucket  = s3.Bucket(old_bucket_name)
        new_bucket = s3.Bucket(new_bucket_name)        
        for obj in old_bucket.objects.filter(Prefix=old_prefix):
            old_source = { 'Bucket': old_bucket_name,
                        'Key': obj.key}
            if old_prefix.endswith('/'):
                new_key = obj.key.replace(old_prefix, new_prefix)
            else:
                new_key = new_prefix + old_prefix.split('/')[-1]
            new_obj = new_bucket.Object(new_key)
            new_obj.copy(old_source)
    except ClientError as e:
        raise S3OperationError(str(e))

def upload_file_from_path(local_file_path, bucket_name, s3_path):
    """
    uploads a file from local path to s3_path

    Parameters
    ----------
    local_file_path: string
        path of the file which needs to be uploaded
    bucket_name: string
        name of the bucket to which file will be uploaded
    s3_path: string
        path where file needs to be uploaded
    """
    try:
        client = boto3.client('s3')
        if s3_path.endswith('/'):
            s3_path = s3_path + local_file_path.split('/')[-1]
        client.upload_file(local_file_path, bucket_name, s3_path)
    except ClientError as e:
        raise S3OperationError(str(e))


def move_object(src_bucket, src_key, des_bucket, des_key):
    """
    This function moves the s3 objects

    Parameters
    -----------
    src_bucket: str
        Source bucket name of the file
    des_bucket: str
        Destination bucket name of the file
    src_key: str
        Source key of the file
    des_key: str
        Destination key of the file
    
    Returns
    --------
    command_exit_code: int
        command exit code
    """
    cmd = "aws s3 mv s3://{}/{} s3://{}/{}".format(src_bucket, 
          src_key, des_bucket, des_key)
    try:
        command_exit_code = subprocess.call(cmd, shell=True)
    except Exception as error:
        raise OSError(str(error))
    return command_exit_code


def delete_files(file_delete_list, bucket):
    """
    This function deletes the files from S3 bucket

    PARAMETERS
    -----------
    file_delete_list: list
        list of files to be deleted
    bucket: string
        bucket name

    RETURNS
    --------
    response: dict
        dictionary object with deleted key
    """
    try:
        s3 = boto3.resource('s3')
        bucket  = s3.Bucket(bucket)
        response = bucket.delete_objects(
            Delete={
                'Objects': file_delete_list
            }
        )
    except ClientError as e:
        raise S3OperationError(str(e))
    return response